@@ -3,21 +3,22 @@ from functools import total_ordering
 from collections import Iterable
 
 
-__version__ = '0.5.2'
-__all__ = ['MultiValueEnum', 'no_overlap', 'StrEnum', 'CaseInsensitiveStrEnum']
+__version__ = '0.6.4'
+__all__ = ['MultiValueEnum', 'no_overlap', 'StrEnum', 'CaseInsensitiveStrEnum',
+           'CaseInsensitiveMultiValueEnum']
 
 
 class _MultiValueMeta(EnumMeta):
     def __init__(self, clsname, bases, classdict):
         # make sure we only have tuple values, not single values
         for member in self.__members__.values():
-            val = member._value_
-            if not isinstance(val, Iterable) or isinstance(val, str):
+            value = member._value_
+            if not isinstance(value, Iterable) or isinstance(value, str):
                 raise TypeError('{} = {!r}, should be iterable, not {}!'
-                    .format(member._name_, val, type(val))
+                    .format(member._name_, value, type(value))
                 )
             # set is faster to lookup
-            member._lookup_set_ = set(val)
+            member._lookup_set_ = set(value)
 
     def __call__(cls, value):
         """Return the appropriate instance with any of the values listed."""
@@ -25,7 +26,8 @@ class _MultiValueMeta(EnumMeta):
             if value in member._lookup_set_:
                 return member
         else:
-            raise ValueError("%s is not a valid %s" % (value, cls.__name__))
+            # lookup by original value, enum instance, or raise ValueError
+            return super().__call__(value)
 
 
 class MultiValueEnum(Enum, metaclass=_MultiValueMeta):
@@ -34,6 +36,41 @@ class MultiValueEnum(Enum, metaclass=_MultiValueMeta):
     """
 
 
+class _CasInsensitiveMultiValueMeta(EnumMeta):
+    def __init__(self, clsname, bases, classdict):
+        # make sure we only have tuple values, not single values
+        for member in self.__members__.values():
+            value = member._value_
+            if not isinstance(value, Iterable) or isinstance(value, str):
+                raise TypeError('{} = {!r}, should be iterable, not {}!'
+                    .format(member._name_, value, type(value))
+                )
+            # set is faster to lookup
+            member._lookup_set_ = set()
+            for elem in value:
+                if isinstance(elem, str):
+                    elem = elem.upper()
+                member._lookup_set_.add(elem)
+
+    def __call__(cls, value):
+        """Return the appropriate instance with any of the values listed."""
+        compare = value.upper() if isinstance(value, str) else value
+        for member in cls:
+            if compare in member._lookup_set_:
+                return member
+        else:
+            # lookup by original value, enum instance, or raise ValueError
+            return super().__call__(value)
+
+
+class CaseInsensitiveMultiValueEnum(
+    Enum, metaclass=_CasInsensitiveMultiValueMeta):
+    """Same as MultiValueEnum, except when member value contains an str,
+    they will be compared in a case-insensitive manner. Non-str types left
+    untouched.
+    """
+
+
 def no_overlap(multienum):
     """Class decorator for MultiValueEnum ensuring non overlapping elements
     in member values. Other words: ensure no element in any member value is
@@ -124,6 +161,17 @@ class OrderableMixin:
     """Mixin for comparable Enums. The order is the definition order
     from smaller to bigger.
     """
+    # From Python manual:
+    # If a class that overrides __eq__() needs to retain
+    # the implementation of __hash__() from a parent class,
+    # the interpreter must be told this explicitly
+    def __hash__(self):
+        return super().__hash__()
+
+    def __reduce_ex__(self, proto):
+        # this will never run anyway, but the Enum class needs it
+        return self.__qualname__
+
     def __eq__(self, other):
         if self.__class__ is other.__class__:
             return self._value_ == other._value_
